@@ -4,6 +4,7 @@ use warnings;
 use strict;
 
 use DateTime::Format::Strptime;
+use DateTime::Format::DateParse;
 use File::Basename qw(basename);
 use File::Path qw(mkpath);
 use HTML::Entities qw(decode_entities);
@@ -27,7 +28,7 @@ CPAN::Testers::ParseReport - parse reports to www.cpantesters.org from various s
 
 =cut
 
-use version; our $VERSION = qv('0.0.18');
+use version; our $VERSION = qv('0.0.19');
 
 =head1 SYNOPSIS
 
@@ -49,15 +50,15 @@ the functions unaltered.
 
 =head1 FUNCTIONS
 
-=head2 parse_distro($distro,$options)
+=head2 parse_distro($distro,%options)
 
 reads the cpantesters HTML page or the YAML file for the distro and
 loops through the reports for the specified or most recent version of
 that distro found in these data.
 
-=head2 parse_single_report($report,$dumpvars,$options)
+=head2 parse_single_report($report,$dumpvars,%options)
 
-mirrors and reads this report. $report is of the for
+mirrors and reads this report. $report is of the form
 
   { id => number }
 
@@ -362,12 +363,20 @@ a hashref which gets filled. %Opt are the options as described in the
 C<ctgetreports> manpage.
 
 Note: this parsing is a bit dirty but as it seems good enough I'm not
-inclined to change it. We parse HTML with a regexps only, no HTML
-parser working, only the entities are decoded.
+inclined to change it. We parse HTML with regexps only, not an HTML
+parser. Only the entities are decoded.
 
 Update around version 0.0.17: switching to nntp now but keeping the
 parser for HTML around to read old local copies.
 
+Update around 0.0.18: In %Options you can use
+
+    article => $some_full_article_as_scalar
+
+to use this function to parse one full article as text. When this is
+given, the argument $target is not read, but its basename is taken to
+be the id of the article. (OMG, hackers!)
+
 =cut
 sub parse_report {
     my($target,$dumpvars,%Opt) = @_;
@@ -378,14 +387,20 @@ sub parse_report {
     my(%extract);
 
     my($report,$isHTML);
-    my @qr = map /^qr:(.+)/, @{$Opt{q}};
-    if ($Opt{raw} || @qr) {
+    if ($report = $Opt{article}) {
+        $isHTML = $report =~ /^</;
+        undef $target;
+    }
+    if ($target) {
         open my $fh, $target or die "Could not open '$target': $!";
         local $/;
         my $raw_report = <$fh>;
         $isHTML = $raw_report =~ /^</;
         $report = $isHTML ? decode_entities($raw_report) : $raw_report;
         close $fh;
+    }
+    my @qr = map /^qr:(.+)/, @{$Opt{q}};
+    if ($Opt{raw} || @qr) {
         for my $qr (@qr) {
             my $cqr = eval "qr{$qr}";
             die "Could not compile regular expression '$qr': $@" if $@;
@@ -404,8 +419,6 @@ sub parse_report {
         }
     }
 
-    open my $fh, $target or die "Could not open '$target': $!";
-
     my $report_writer;
     my $moduleunpack = {};
     my $expect_prereq = 0;
@@ -418,8 +431,8 @@ sub parse_report {
 
     my $current_headline;
     my @previous_line = ""; # so we can neutralize line breaks
- LINE: while (<$fh>) {
-        $isHTML = /^</ if ! defined $isHTML && $. == 1;
+    my @rlines = split /\r?\n/, $report;
+ LINE: for (@rlines) {
         next LINE unless ($isHTML ? m/<title>(\S+)\s+(\S+)/ : m/^Subject: (\S+)\s+(\S+)/);
         $ok = $1;
         $about = $2;
@@ -427,16 +440,13 @@ sub parse_report {
         $extract{"meta:about"} = $about;
         last;
     }
-    seek $fh, 0, 0;
- LINE: while (<$fh>) {
-        s/\r?\n\z//;
+ LINE: while (@rlines) {
+        $_ = shift @rlines;
         while (/!$/) {
-            my $followupline = <$fh>;
+            my $followupline = shift @rlines;
             $followupline =~ s/^\s+//; # remo leading space
             $_ .= $followupline;
-            s/\r?\n\z//;
         }
-        $_ = decode_entities($_) if $isHTML;
         if (/^--------/ && $previous_line[-2] && $previous_line[-2] =~ /^--------/) {
             $current_headline = $previous_line[-1];
             if ($current_headline =~ /PROGRAM OUTPUT/) {
@@ -491,23 +501,21 @@ sub parse_report {
                      m|^Date: (.+)|
                     ) {
                 my $date = $1;
-                my $p;
+                my($dt);
                 if ($isHTML) {
+                    my $p;
                     $p = DateTime::Format::Strptime->new(
                                                          locale => "en",
                                                          time_zone => "UTC",
                                                          # April 13, 2005 23:50
                                                          pattern => "%b %d, %Y %R",
                                                         );
+                    $dt = $p->parse_datetime($date);
                 } else {
-                    $p = DateTime::Format::Strptime->new(
-                                                         locale => "en",
-                                                         time_zone => "UTC",
-                                                         # Sun, 28 Sep 2008 12:23:12 +0100
-                                                         pattern => "%a, %d %b %Y %T %z",
-                                                        );
+                    # Sun, 28 Sep 2008 12:23:12 +0100 # but was not consistent
+                    # pattern => "%a, %d %b %Y %T %z",
+                    $dt = DateTime::Format::DateParse->parse_datetime($date);
                 }
-                my $dt = $p->parse_datetime($date);
                 $extract{"meta:date"} = $dt->datetime;
             }
             $extract{"meta:date"} =~ s/\.$// if $extract{"meta:date"};
